from collections import OrderedDict
from czsc import CZSC as CZSC
from czsc.core import BI as BI, Direction as Direction, FX as FX, Mark as Mark, ZS as ZS
from czsc.signals.tas import update_ma_cache as update_ma_cache, update_macd_cache as update_macd_cache
from czsc.traders.base import CzscSignals as CzscSignals
from czsc.utils.sig import create_single_signal as create_single_signal, get_sub_elements as get_sub_elements, get_zs_seq as get_zs_seq

def cxt_bi_base_V230228(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_fx_power_V221107(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_first_buy_V221126(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_first_sell_V221126(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_zhong_shu_gong_zhen_V221221(cat: CzscSignals, freq1: str = '日线', freq2: str = '60分钟', **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230222(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230224(c: CZSC, **kwargs): ...
def cxt_third_buy_V230228(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_double_zs_V230311(c: CZSC, **kwargs): ...
def cxt_second_bs_V230320(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_third_bs_V230318(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_third_bs_V230319(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230104(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230105(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230312(c: CZSC, **kwargs): ...
def cxt_bi_end_V230320(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230322(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230324(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_status_V230101(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_status_V230102(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_zdf_V230601(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230618(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_three_bi_V230618(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_five_bi_V230619(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_seven_bi_V230620(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_nine_bi_V230621(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_eleven_bi_V230622(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_range_oscillation_V230620(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_intraday_V230701(cat: CzscSignals, **kwargs) -> OrderedDict: ...
def cxt_ubi_end_V230816(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230815(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_stop_V230815(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_trend_V230824(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_trend_V230913(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_second_bs_V240524(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_overlap_V240526(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_decision_V240526(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bs_V240526(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bs_V240527(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_overlap_V240612(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_decision_V240612(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_decision_V240613(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_decision_V240614(c: CZSC, **kwargs) -> OrderedDict: ...
