import pandas as pd
from .base import apply_stats_style as apply_stats_style, generate_component_key as generate_component_key, safe_import_weight_backtest as safe_import_weight_backtest
from loguru import logger as logger

def show_optuna_study(study, key=None, **kwargs): ...
def show_czsc_trader(trader, max_k_num: int = 300, key=None, **kwargs): ...
def show_strategies_recent(df, **kwargs): ...
def show_returns_contribution(df, returns=None, max_returns: int = 100, key=None) -> None: ...
def show_symbols_bench(df: pd.DataFrame, **kwargs): ...
def show_quarterly_effect(returns: pd.Series, key=None): ...
def show_multi_backtest(wbs: dict, **kwargs): ...
def show_cta_periods_classify(df: pd.DataFrame, **kwargs): ...
def show_volatility_classify(df: pd.DataFrame, kind: str = 'ts', **kwargs): ...
def show_portfolio(df: pd.DataFrame, portfolio: str, benchmark: str | None = None, **kwargs): ...
def show_turnover_rate(df: pd.DataFrame): ...
def show_stats_compare(df: pd.DataFrame, **kwargs): ...
def show_symbol_penalty(df: pd.DataFrame, n: int = 3, **kwargs): ...
