import pandas as pd
from .base import safe_import_daily_performance as safe_import_daily_performance, safe_import_weight_backtest as safe_import_weight_backtest

def show_weight_distribution(dfw, abs_weight: bool = True, **kwargs): ...
def show_weight_backtest(dfw, **kwargs): ...
def show_holds_backtest(df, **kwargs) -> None: ...
def show_stoploss_by_direction(dfw, **kwargs) -> None: ...
def show_backtest_by_thresholds(df: pd.DataFrame, out_sample_sdt, **kwargs): ...
def show_backtest_by_year(df: pd.DataFrame, **kwargs): ...
def show_backtest_by_symbol(df: pd.DataFrame, **kwargs): ...
def show_long_short_backtest(df: pd.DataFrame, **kwargs): ...
def show_comprehensive_weight_backtest(df: pd.DataFrame, **kwargs): ...
