from collections import OrderedDict
from czsc.core import Direction as Direction, Mark as Mark, Operate as Operate, ZS as ZS
from czsc.signals.tas import update_macd_cache as update_macd_cache
from czsc.traders.base import CZSC as CZSC, CzscTrader as CzscTrader
from czsc.utils import sorted_freqs as sorted_freqs
from czsc.utils.sig import create_single_signal as create_single_signal, get_sub_elements as get_sub_elements
from loguru import logger as logger

def zdy_bi_end_V230406(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_bi_end_V230407(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_vibrate_V230406(cat: CzscTrader, freq1, freq2, **kwargs) -> OrderedDict: ...
def zdy_stop_loss_V230406(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def zdy_take_profit_V230406(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def zdy_take_profit_V230407(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def zdy_zs_V230423(c: CZSC, **kwargs): ...
def zdy_macd_bc_V230422(c: CZSC, **kwargs): ...
def zdy_macd_bs1_V230422(c: CZSC, **kwargs): ...
def zdy_zs_space_V230421(c: CZSC, **kwargs): ...
def zdy_macd_dif_V230516(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_macd_dif_V230517(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_macd_V230518(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_macd_V230519(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_macd_dif_iqr_V230521(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_macd_V230527(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_dif_V230527(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_dif_V230528(c: CZSC, **kwargs) -> OrderedDict: ...
def pressure_support_V240222(c: CZSC, **kwargs) -> OrderedDict: ...
def pressure_support_V240402(c: CZSC, **kwargs) -> OrderedDict: ...
def pressure_support_V240406(c: CZSC, **kwargs) -> OrderedDict: ...
def pressure_support_V240530(c: CZSC, **kwargs) -> OrderedDict: ...
