import pandas as pd

def validate_dataframe_columns(df: pd.DataFrame, required_columns: list[str], name: str = 'DataFrame') -> None: ...
def validate_datetime_index(df: pd.DataFrame, name: str = 'DataFrame') -> None: ...
def validate_numeric_column(df: pd.DataFrame, column: str, name: str = 'DataFrame') -> None: ...
def validate_date_range(df: pd.DataFrame, start_date: pd.Timestamp | None = None, end_date: pd.Timestamp | None = None, name: str = 'DataFrame') -> None: ...
def validate_no_duplicates(df: pd.DataFrame, subset: list[str] | None = None, name: str = 'DataFrame') -> None: ...
def validate_weight_data(df: pd.DataFrame) -> None: ...
