from collections import OrderedDict
from czsc.core import CZSC as CZSC
from czsc.signals.jcc import check_szx as check_szx
from czsc.signals.tas import update_ma_cache as update_ma_cache
from czsc.utils.sig import create_single_signal as create_single_signal, get_sub_elements as get_sub_elements

def xl_bar_position_V240328(c: CZSC, **kwargs) -> OrderedDict: ...
def xl_bar_trend_V240329(c: CZSC, **kwargs) -> OrderedDict: ...
def xl_bar_trend_V240330(c: CZSC, **kwargs) -> OrderedDict: ...
def xl_bar_trend_V240331(c: CZSC, **kwargs) -> OrderedDict: ...
def xl_bar_basis_V240412(c: CZSC, **kwargs) -> OrderedDict: ...
def xl_bar_basis_V240411(c: CZSC, **kwargs) -> OrderedDict: ...
def xl_bar_trend_V240623(c: CZSC, **kwargs) -> OrderedDict: ...
