from _typeshed import Incomplete
from pathlib import Path
from typing import Any, AnyStr

home_path: Incomplete

def get_dir_size(path): ...
def empty_cache_path() -> None: ...

class DiskCache:
    path: Incomplete
    def __init__(self, path=None) -> None: ...
    def is_found(self, k: str, suffix: str = 'pkl', ttl: int = -1) -> bool: ...
    def get(self, k: str, suffix: str = 'pkl') -> Any: ...
    def set(self, k: str, v: Any, suffix: str = 'pkl'): ...
    def remove(self, k: str, suffix: str = 'pkl'): ...

def disk_cache(path: AnyStr | Path = ..., suffix: str = 'pkl', ttl: int = -1): ...
def clear_expired_cache(path, max_age: int = ...): ...
def clear_cache(path: AnyStr | Path = ..., subs=None, recreate: bool = False): ...
