import numpy as np
import pandas as pd

def nmi_matrix(df: pd.DataFrame, heatmap: bool = False) -> pd.DataFrame: ...
def single_linear(y: np.array | list, x: np.array | list = None) -> dict: ...
def cross_sectional_ic(df, x_col: str = 'open', y_col: str = 'n1b', method: str = 'spearman', **kwargs): ...
