import pandas as pd
from _typeshed import Incomplete
from czsc.py.objects import Freq as Freq, RawBar as RawBar
from datetime import datetime
from typing import AnyStr

freq_market_times: Incomplete
freq_edt_map: Incomplete

def is_trading_time(dt: datetime = ..., market: str = 'A股'): ...
def get_intraday_times(freq: str = '1分钟', market: str = 'A股'): ...
def format_standard_kline(df: pd.DataFrame, freq: str): ...
def check_freq_and_market(time_seq: list[AnyStr], freq: AnyStr | None = None): ...
def freq_end_date(dt, freq: Freq | AnyStr): ...
def freq_end_time(dt: datetime, freq: Freq | AnyStr, market: str = 'A股') -> datetime: ...
def resample_bars(df: pd.DataFrame, target_freq: Freq | AnyStr, raw_bars: bool = True, **kwargs): ...

class BarGenerator:
    version: str
    symbol: Incomplete
    end_dt: Incomplete
    market: Incomplete
    base_freq: Incomplete
    max_count: Incomplete
    freqs: Incomplete
    bars: Incomplete
    freq_map: Incomplete
    def __init__(self, base_freq: str, freqs: list[str], max_count: int = 5000, market: str = '默认') -> None: ...
    def init_freq_bars(self, freq: str, bars: list[RawBar]): ...
    def update(self, bar: RawBar) -> None: ...
