from _typeshed import Incomplete

calendar: Incomplete

def prepare_chain_calendar() -> None: ...
def is_trading_date(date=...): ...
def next_trading_date(date=..., n: int = 1): ...
def prev_trading_date(date=..., n: int = 1): ...
def get_trading_dates(sdt, edt=...): ...
