def RET001(df, **kwargs) -> None: ...
def RET002(df, **kwargs) -> None: ...
def RET003(df, **kwargs) -> None: ...
def RET004(df, **kwargs): ...
def RET005(df, **kwargs): ...
def RET006(df, **kwargs): ...
def RET007(df, **kwargs): ...
def RET008(df, **kwargs): ...
def test_ret_functions() -> None: ...
