from tenacity import retry as retry, stop_after_attempt as stop_after_attempt, wait_fixed as wait_fixed

def get_symbols(exchange: str = '币安期货', **kwargs): ...
def get_raw_bars(symbol: str = 'BTCUSDT', period: str = '4h', sdt: str = '20240101', edt: str = '20240308', **kwargs): ...
def get_latest_klines(symbol, period, sdt=None, **kwargs): ...
def test_fetcher() -> None: ...
