from _typeshed import Incomplete
from czsc.traders.dummy import DummyBacktest as DummyBacktest

class CTAResearch:
    version: str
    strategy: Incomplete
    read_bars: Incomplete
    results_path: Incomplete
    kwargs: Incomplete
    signals_module_name: Incomplete
    def __init__(self, strategy, read_bars, results_path, **kwargs) -> None: ...
    def replay(self, symbol, sdt: str = '20200101', edt: str = '20220101', refresh: bool = True) -> None: ...
    def check_signals(self, symbol, sdt: str = '20200101', edt: str = '20220101') -> None: ...
    def dummy(self, symbols, sdt: str = '20200101', edt: str = '20220101', max_workers: int = 1, **kwargs) -> None: ...
    def backtest(self, symbols, max_workers: int = 3, **kwargs) -> None: ...
