import pandas as pd
from .base import apply_stats_style as apply_stats_style, ensure_datetime_index as ensure_datetime_index, generate_component_key as generate_component_key, safe_import_daily_performance as safe_import_daily_performance

def show_splited_daily(df, ret_col, **kwargs) -> None: ...
def show_yearly_stats(df, ret_col, **kwargs) -> None: ...
def show_out_in_compare(df, ret_col, mid_dt, **kwargs) -> None: ...
def show_outsample_by_dailys(df, outsample_sdt1, outsample_sdt2=None) -> None: ...
def show_psi(df, factor, segment, **kwargs) -> None: ...
def show_classify(df, col1, col2, n: int = 10, method: str = 'cut', key=None, **kwargs): ...
def show_date_effect(df: pd.DataFrame, ret_col: str, **kwargs): ...
def show_normality_check(data: pd.Series, alpha: float = 0.05): ...
def show_describe(df: pd.DataFrame, **kwargs): ...
def show_df_describe(df: pd.DataFrame): ...
