def dill_dump(data, file) -> None: ...
def dill_load(file): ...
def save_pkl(data, file) -> None: ...
def read_pkl(file): ...
def save_json(data, file) -> None: ...
def read_json(file): ...
def make_zip(source_dir: str, file_zip: str) -> None: ...
