from collections import OrderedDict
from czsc.core import CZSC as CZSC, RawBar as RawBar
from czsc.utils.sig import create_single_signal as create_single_signal, get_sub_elements as get_sub_elements

def adtm_up_dw_line_V230603(c: CZSC, **kwargs) -> OrderedDict: ...
def amv_up_dw_line_V230603(c: CZSC, **kwargs) -> OrderedDict: ...
def asi_up_dw_line_V230603(c: CZSC, **kwargs) -> OrderedDict: ...
def clv_up_dw_line_V230605(c: CZSC, **kwargs) -> OrderedDict: ...
def cmo_up_dw_line_V230605(c: CZSC, **kwargs) -> OrderedDict: ...
def skdj_up_dw_line_V230611(c: CZSC, **kwargs) -> OrderedDict: ...
def bias_up_dw_line_V230618(c: CZSC, **kwargs) -> OrderedDict: ...
def dema_up_dw_line_V230605(c: CZSC, **kwargs) -> OrderedDict: ...
def demakder_up_dw_line_V230605(c: CZSC, **kwargs) -> OrderedDict: ...
def emv_up_dw_line_V230605(c: CZSC, **kwargs) -> OrderedDict: ...
def er_up_dw_line_V230604(c: CZSC, **kwargs) -> OrderedDict: ...
def obvm_line_V230610(c: CZSC, **kwargs) -> OrderedDict: ...
def obv_up_dw_line_V230719(c: CZSC, **kwargs) -> OrderedDict: ...
def cvolp_up_dw_line_V230612(c: CZSC, **kwargs) -> OrderedDict: ...
def ntmdk_V230824(c: CZSC, **kwargs) -> OrderedDict: ...
def kcatr_up_dw_line_V230823(c: CZSC, **kwargs) -> OrderedDict: ...
