def generate_fernet_key(): ...
def fernet_encrypt(data: dict | str, key: str = None) -> str: ...
def fernet_decrypt(data: str, key: str = None, is_dict: bool = False) -> str: ...
def test() -> None: ...
