from collections import OrderedDict
from czsc import envs as envs
from czsc.core import CZSC as CZSC, RawBar as RawBar, Signal as Signal, freq_end_time as freq_end_time
from czsc.signals.tas import update_ma_cache as update_ma_cache, update_macd_cache as update_macd_cache
from czsc.utils import single_linear as single_linear
from czsc.utils.sig import create_single_signal as create_single_signal, get_sub_elements as get_sub_elements
from deprecated import deprecated as deprecated

def bar_single_V230506(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_triple_V230506(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_end_V221211(c: CZSC, freq1: str = '60分钟', **kwargs) -> OrderedDict: ...
def bar_operate_span_V221111(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_zdt_V230331(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_vol_grow_V221112(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_fang_liang_break_V221216(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_mean_amount_V221112(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_section_momentum_V221112(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_accelerate_V221110(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_accelerate_V221118(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_accelerate_V240428(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_zdf_V221203(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_fake_break_V230204(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_single_V230214(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_amount_acc_V230214(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_big_solid_V230215(c: CZSC, **kwargs): ...
def bar_vol_bs1_V230224(c: CZSC, **kwargs): ...
def bar_reversal_V230227(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_bpm_V230227(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_time_V230327(c: CZSC, **kwargs): ...
def bar_weekday_V230328(c: CZSC, **kwargs): ...
def bar_r_breaker_V230326(c: CZSC, **kwargs): ...
def bar_dual_thrust_V230403(c: CZSC, **kwargs): ...
def bar_zt_count_V230504(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_channel_V230508(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_tnr_V230630(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_tnr_V230629(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_shuang_fei_V230507(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_limit_down_V230525(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_eight_V230702(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_window_std_V230731(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_window_ps_V230731(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_window_ps_V230801(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_trend_V240209(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_plr_V240427(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_polyfit_V240428(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_break_V240428(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_classify_V240606(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_classify_V240607(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_decision_V240608(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_decision_V240616(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_td9_V240616(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_volatility_V241013(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_zfzd_V241013(c: CZSC, **kwargs) -> OrderedDict: ...
def bar_zfzd_V241014(c: CZSC, **kwargs) -> OrderedDict: ...
