__all__ = ['show_weight_ts', 'show_weight_dist', 'show_weight_cdf', 'show_weight_abs']

def show_weight_ts(df, key=None, **kwargs) -> None: ...
def show_weight_dist(df, key=None, **kwargs) -> None: ...
def show_weight_cdf(df, key=None, **kwargs) -> None: ...
def show_weight_abs(df, key=None, **kwargs) -> None: ...
