from collections import OrderedDict
from czsc.core import CZSC as CZSC, Direction as Direction, Mark as Mark, Operate as Operate
from czsc.signals.tas import update_ma_cache as update_ma_cache
from czsc.traders.base import CzscTrader as CzscTrader
from czsc.utils.sig import create_single_signal as create_single_signal, get_sub_elements as get_sub_elements

def pos_ma_V230414(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_fx_stop_V230414(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_bar_stop_V230524(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_holds_V230414(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_fix_exit_V230624(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_profit_loss_V230624(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_status_V230808(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_holds_V230807(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_holds_V240428(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_holds_V240608(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_stop_V240428(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_take_V240428(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_stop_V240331(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_stop_V240608(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_stop_V240614(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def pos_stop_V240717(cat: CzscTrader, **kwargs) -> OrderedDict: ...
