from _typeshed import Incomplete
from czsc.core import Signal as Signal

class SignalsParser:
    signals_module: Incomplete
    sig_name_map: Incomplete
    sig_pats_map: Incomplete
    def __init__(self, signals_module: str = 'czsc.signals') -> None: ...
    def parse_params(self, name, signal): ...
    def get_function_name(self, signal: str): ...
    def config_to_keys(self, config: list[dict]): ...
    def parse(self, signal_seq: list[str]): ...

def get_signals_config(signals_seq: list[str], signals_module: str = 'czsc.signals') -> list[dict]: ...
def get_signals_freqs(signals_seq: list) -> list[str]: ...
