import pandas as pd

def risk_free_returns(start_date: str = '20180101', end_date: str = '20210101', year_returns: float = 0.03): ...
def update_nxb(df: pd.DataFrame, **kwargs) -> pd.DataFrame: ...
def update_bbars(da, price_col: str = 'close', numbers=(1, 2, 5, 10, 20, 30)) -> None: ...
def update_tbars(da: pd.DataFrame, event_col: str) -> None: ...
def resample_to_daily(df: pd.DataFrame, sdt=None, edt=None, only_trade_date: bool = True): ...
def adjust_holding_weights(df, hold_periods: int = 1, **kwargs): ...
