import pandas as pd

def normalize_feature(df, x_col, method: str = 'standard', **kwargs): ...
def normalize_ts_feature(df, x_col, n: int = 10, **kwargs): ...
def feature_cross_layering(df, x_col, **kwargs): ...
def find_most_similarity(vector: pd.Series, matrix: pd.DataFrame, n: int = 10, metric: str = 'cosine', **kwargs): ...
