import pandas as pd

def is_event_feature(df, col, **kwargs): ...
def rolling_corr(df, col1, col2, window: int = 300, min_periods: int = 100, **kwargs): ...
def rolling_rank(df: pd.DataFrame, col, window: int = 300, min_periods: int = 100, new_col=None, **kwargs): ...
def rolling_norm(df: pd.DataFrame, col, window: int = 300, min_periods: int = 100, new_col=None, **kwargs): ...
def rolling_qcut(df: pd.DataFrame, col, window: int = 300, min_periods: int = 100, new_col=None, **kwargs): ...
def rolling_compare(df, col1, col2, window: int = 300, min_periods: int = 100, new_col=None, **kwargs) -> None: ...
def rolling_scale(df: pd.DataFrame, col: str, window: int = 300, min_periods: int = 100, new_col=None, **kwargs): ...
def rolling_tanh(df: pd.DataFrame, col: str, window: int = 300, min_periods: int = 100, new_col=None, **kwargs): ...
def rolling_slope(df: pd.DataFrame, col: str, window: int = 300, min_periods: int = 100, new_col=None, **kwargs): ...
def normalize_corr(df: pd.DataFrame, fcol, ycol=None, **kwargs): ...
def feature_returns(df, factor, target: str = 'n1b', **kwargs): ...
def feature_sectional_corr(df, factor, target: str = 'n1b', method: str = 'pearson', **kwargs): ...
