from czsc.utils.data.cache import disk_cache as disk_cache

def generate_symbol_kines(symbol, freq, sdt: str = '20100101', edt: str = '20250101', seed: int = 42): ...
def generate_klines(seed: int = 42): ...
def generate_klines_with_weights(seed: int = 42): ...
def generate_ts_factor(seed: int = 42): ...
def generate_cs_factor(seed: int = 42): ...
def generate_strategy_returns(n_strategies: int = 10, n_days=None, seed: int = 42): ...
def generate_portfolio(seed: int = 42): ...
def set_global_seed(seed: int = 42) -> None: ...
def generate_correlation_data(seed: int = 42): ...
def generate_daily_returns(n_strategies: int = 3, seed: int = 42): ...
def generate_statistics_data(seed: int = 42): ...
def generate_event_data(seed: int = 42): ...
