import pandas as pd

def cal_break_even_point(seq) -> float: ...
def daily_performance(daily_returns, **kwargs): ...
def rolling_daily_performance(df: pd.DataFrame, ret_col, window: int = 252, min_periods: int = 100, **kwargs): ...
def evaluate_pairs(pairs: pd.DataFrame, trade_dir: str = '多空') -> dict: ...
def holds_performance(df, **kwargs): ...
def top_drawdowns(returns: pd.Series, top: int = 10) -> pd.DataFrame: ...
def psi(df: pd.DataFrame, factor, segment, **kwargs): ...
