import pandas as pd

def max_draw_down(n1b: list): ...
def turn_over_rate(df_holds: pd.DataFrame) -> tuple[pd.DataFrame, float]: ...
def holds_concepts_effect(holds: pd.DataFrame, concepts: dict, top_n: int = 20, min_n: int = 3, **kwargs): ...
