import pandas as pd
from _typeshed import Incomplete

class PairsPerformance:
    df_pairs: Incomplete
    agg_columns: Incomplete
    def __init__(self, df_pairs: pd.DataFrame) -> None: ...
    @staticmethod
    def get_pairs_statistics(df_pairs: pd.DataFrame): ...
    def agg_statistics(self, col: str): ...
    @property
    def basic_info(self): ...
    def agg_to_excel(self, file_xlsx) -> None: ...

def combine_holds_and_pairs(holds, pairs, results_path): ...
def combine_dates_and_pairs(dates: list, pairs: pd.DataFrame, results_path): ...
