from collections import OrderedDict
from czsc.core import CZSC as CZSC, RawBar as RawBar
from czsc.utils.sig import create_single_signal as create_single_signal, get_sub_elements as get_sub_elements

def update_vol_ma_cache(c: CZSC, ma_type: str, timeperiod: int, **kwargs): ...
def vol_single_ma_V230214(c: CZSC, **kwargs) -> OrderedDict: ...
def vol_double_ma_V230214(c: CZSC, **kwargs) -> OrderedDict: ...
def vol_ti_suo_V221216(c: CZSC, **kwargs) -> OrderedDict: ...
def vol_gao_di_V221218(c: CZSC, **kwargs) -> OrderedDict: ...
def vol_window_V230731(c: CZSC, **kwargs) -> OrderedDict: ...
def vol_window_V230801(c: CZSC, **kwargs) -> OrderedDict: ...
