def safe_import_daily_performance(): ...
def safe_import_weight_backtest(): ...
def safe_import_top_drawdowns(): ...
def apply_stats_style(stats_df): ...
def ensure_datetime_index(df, dt_col: str = 'dt'): ...
def generate_component_key(data, prefix: str = 'component', **kwargs): ...
