from collections import OrderedDict
from czsc.core import BI as BI, CZSC as CZSC, Direction as Direction, FX as FX, Mark as Mark, RawBar as RawBar, Signal as Signal, ZS as ZS
from czsc.traders.base import CzscSignals as CzscSignals
from czsc.utils import single_linear as single_linear
from czsc.utils.sig import cal_cross_num as cal_cross_num, count_last_same as count_last_same, create_single_signal as create_single_signal, cross_zero_axis as cross_zero_axis, down_cross_count as down_cross_count, fast_slow_cross as fast_slow_cross, get_sub_elements as get_sub_elements
from deprecated import deprecated as deprecated

def update_ma_cache(c: CZSC, **kwargs): ...
def update_macd_cache(c: CZSC, **kwargs): ...
def update_boll_cache_V230228(c: CZSC, **kwargs): ...
def update_boll_cache(c: CZSC, **kwargs): ...
def tas_boll_vt_V230212(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_base_V221028(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_direct_V221106(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_power_V221108(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_first_bs_V221201(c: CZSC, **kwargs): ...
def tas_macd_first_bs_V221216(c: CZSC, **kwargs): ...
def tas_macd_second_bs_V221201(c: CZSC, **kwargs): ...
def tas_macd_xt_V221208(c: CZSC, **kwargs): ...
def tas_macd_bc_V221201(c: CZSC, **kwargs): ...
def tas_macd_change_V221105(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_ma_base_V221101(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_ma_base_V221203(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_ma_base_V230313(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_ma_round_V221206(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_double_ma_V221203(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_double_ma_V230511(c: CZSC, **kwargs): ...
def tas_boll_power_V221112(c: CZSC, **kwargs): ...
def tas_boll_bc_V221118(c: CZSC, **kwargs): ...
def update_kdj_cache(c: CZSC, **kwargs): ...
def tas_kdj_base_V221101(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_kdj_evc_V221201(c: CZSC, **kwargs) -> OrderedDict: ...
def update_rsi_cache(c: CZSC, **kwargs): ...
def tas_rsi_base_V230227(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_first_bs_V230217(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_second_bs_V230228(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_second_bs_V230303(c: CZSC, **kwargs): ...
def tas_hlma_V230301(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_boll_cc_V230312(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bs1_V230312(c: CZSC, **kwargs): ...
def tas_macd_bs1_V230313(c: CZSC, **kwargs): ...
def tas_macd_base_V230320(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_dif_layer_V241010(c: CZSC, **kwargs) -> OrderedDict: ...
def update_cci_cache(c: CZSC, **kwargs): ...
def tas_cci_base_V230402(c: CZSC, **kwargs) -> OrderedDict: ...
def cci_decision_V240620(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_kdj_evc_V230401(c: CZSC, **kwargs) -> OrderedDict: ...
def update_atr_cache(c: CZSC, **kwargs): ...
def tas_atr_break_V230424(c: CZSC, **kwargs): ...
def update_sar_cache(c: CZSC, **kwargs): ...
def tas_sar_base_V230425(c: CZSC, **kwargs): ...
def tas_ma_system_V230513(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bs1_V230411(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bs1_V230412(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_accelerate_V230531(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_ma_cohere_V230512(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_cross_status_V230619(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_cross_status_V230624(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_cross_status_V230625(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_low_trend_V230627(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_atr_V230630(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_rumi_V230704(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_dist_V230408(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_dist_V230409(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_dist_V230410(c: CZSC, **kwargs) -> OrderedDict: ...
def cat_macd_V230518(cat: CzscSignals, **kwargs) -> OrderedDict: ...
def cat_macd_V230520(cat: CzscSignals, **kwargs) -> OrderedDict: ...
def tas_angle_V230802(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bc_V230803(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bc_V230804(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bc_ubi_V230804(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_slope_V231019(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_double_ma_V240208(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bc_V240307(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_dma_bs_V240608(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_dif_zero_V240612(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_dif_zero_V240614(c: CZSC, **kwargs) -> OrderedDict: ...
