import pandas as pd
from _typeshed import Incomplete

class WordWriter:
    file_docx: Incomplete
    document: Incomplete
    def __init__(self, file_docx=None) -> None: ...
    def add_title(self, text) -> None: ...
    def add_heading(self, text, level: int = 1) -> None: ...
    def add_paragraph(self, text, style=None, bold: bool = False, first_line_indent: float = 0.74) -> None: ...
    def add_df_table(self, df: pd.DataFrame, style: str = 'Table Grid', **kwargs): ...
    def add_picture(self, file, width=None, height=None, alignment: str = 'center') -> None: ...
    def add_page_break(self) -> None: ...
    def save(self, file_docx=None) -> None: ...

def test_word_writer() -> None: ...
