from czsc.fsa.base import FeishuApiBase as FeishuApiBase, request as request

class IM(FeishuApiBase):
    def __init__(self, app_id, app_secret, **kwargs) -> None: ...
    def get_user_id(self, payload, user_id_type: str = 'open_id'): ...
    def upload_im_file(self, file_path, file_type: str = 'stream'): ...
    def upload_im_image(self, image_path, image_type: str = 'message'): ...
    def send(self, payload, receive_id_type: str = 'open_id'): ...
    def send_text(self, text, receive_id, receive_id_type: str = 'open_id'): ...
    def send_card(self, card, receive_id, receive_id_type: str = 'open_id'): ...
    def send_image(self, image_path, receive_id, receive_id_type: str = 'open_id'): ...
    def send_file(self, file_path, receive_id, receive_id_type: str = 'open_id'): ...
