from collections import OrderedDict
from czsc import CZSC as CZSC
from czsc.core import Direction as Direction, RawBar as RawBar, Signal as Signal
from czsc.utils.sig import get_sub_elements as get_sub_elements

def jcc_san_xing_xian_V221023(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_ten_mo_V221028(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_wu_yun_gai_ding_V221101(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_ci_tou_V221101(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_san_fa_V20221118(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_san_fa_V20221115(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_xing_xian_V221118(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_fen_shou_xian_V20221113(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_zhu_huo_xian_V221027(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_yun_xian_V221118(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_ping_tou_V221113(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_zhuo_yao_dai_xian_v221113(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_two_crow_V221108(c: CZSC, **kwargs): ...
def jcc_three_crow_V221108(c: CZSC, **kwargs): ...
def check_szx(bar: RawBar, th: int = 10, **kwargs) -> bool: ...
def jcc_szx_V221111(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_san_szx_V221122(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_fan_ji_xian_V221121(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_shan_chun_V221121(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_gap_yin_yang_V221121(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_ta_xing_V221124(c: CZSC, **kwargs) -> OrderedDict: ...
