import pandas as pd
from _typeshed import Incomplete
from czsc import Freq as Freq, RawBar as RawBar

cache_path: Incomplete
dc: Incomplete

def format_kline(kline: pd.DataFrame, freq: Freq) -> list[RawBar]: ...
def get_sw_members(level: str = 'L1'): ...
def get_daily_basic(sdt: str = '20100101', edt: str = '20240101'): ...
def moneyflow_hsgt(start_date, end_date): ...
def pro_bar_minutes(ts_code, sdt, edt, freq: str = '60min', asset: str = 'E', adj=None): ...
def get_symbols(step: str = 'all'): ...
def get_raw_bars(symbol, freq, sdt, edt, fq: str = '后复权', raw_bar: bool = True): ...
