import pandas as pd
from _typeshed import Incomplete

def set_url_token(token, url, **kwargs) -> None: ...
def get_url_token(url, **kwargs): ...

class DataClient:
    __version__: str
    logger: Incomplete
    cache_path: Incomplete
    verbose: Incomplete
    def __init__(self, token: str | None = None, url: str = 'http://api.tushare.pro', timeout: int = 300, verbose: bool = False, **kwargs) -> None: ...
    def clear_cache(self, **kwargs) -> None: ...
    def clear_api_cache(self, api_name: str, expiration: int = -1) -> None: ...
    def post_request(self, api_name: str, fields: str = '', **kwargs) -> pd.DataFrame: ...
    def __getattr__(self, name): ...
