from _typeshed import Incomplete

valid_true: Incomplete

def use_python(): ...
def get_verbose(verbose=None): ...
def get_welcome(): ...
def get_min_bi_len(v: int = None) -> int: ...
def get_max_bi_num(v: int = None) -> int: ...
